package com.bernd32.jlyrics.database;

import androidx.lifecycle.LiveData;
import androidx.room.Dao;
import androidx.room.Delete;
import androidx.room.Insert;
import androidx.room.OnConflictStrategy;
import androidx.room.Query;

import java.util.List;

@Dao
public interface LyricsDao {

    // LiveData is a data holder class that can be observed within a given lifecycle.
    // Always holds/caches latest version of data. Notifies its active observers when the
    // data has changed. Since we are getting all the contents of the database,
    // we are notified whenever any of the database contents have changed.
    @Query("SELECT * from lyrics ORDER BY id DESC")
    LiveData<List<FavLyrics>> getFavLyricsById();

    @Insert(onConflict = OnConflictStrategy.IGNORE)
    void insert(FavLyrics lyrics);

    @Query("DELETE FROM lyrics")
    void deleteAll();

    @Delete
    void deleteLyrics(FavLyrics lyrics);

    @Query("SELECT count(*) FROM lyrics WHERE url = :url")
    int isLyricsExists(String url);

    @Query("DELETE FROM lyrics WHERE url = :url")
    void deleteLyricsByUrl(String url);
}
